/* DASubprocess.h created by DAspinall on Thu 27-Aug-1998 */

/*
 DASubprocess: a wrapper class for NSTask execution.

 History: version 1.0 released Aug 31, 1998

 Motivation:
    I started by using Scott Anguish's category of NSTask from the Rhapsody/OpenStep application called OpenUp version 1.01

    My problem was that I wanted to capture both standard output and standard error, and Scott's category only captured one.  I also wanted to manage better exception handling, since I was not always guaranteed that the command would be entered correctly.

    So I expanded that category to capture both from pipes and to catch exceptions during NSTask launch.  The modified source is included.

    Unfortunately, I experienced a problem during testing where a simple command like "ls c:/winnt/system32" would hang and not complete like it should.  After more testing and 'gasp' reading the documentation, I believe the [NSFileHandle -availableData] command was the culprit.  I think the pipe would fill to the WinNT BUFFER and the subprocess would suspend waiting for the parent to read the data.  However, after the data was read, the sub-task did not restart.  I could be wrong about this, so please feel free to test it yourself.  It may also be a WO 3.51 thing ...

    So, I decided to subclass NSTask, but it is a Class Cluster (surprise!), so this seemed more trouble than it was worth.  So I created DASubprocess instead.   I took a hard look at Mike Ferris's "TEPipeCommand" from the Text Extra's project, and frankly I stole several ideas from it.  However I feel that DASubprocess is different enough to warrant a release of my own.

    For one thing, DASubprocess covers almost the complete NSTask api, so you should be able to plug DASubprocess in anywhere you have NSTask with minimal code change.

    The final thing I wanted this class to do, was to support pipe'd commands. For example "ls | wc -l" should return the count of lines from the ls command.  A more efficient solution would be to serialize the NSTasks and feed the output pipe as input to the next process, but this implementation meets my current needs.
*/
#import <WebObjects/WebObjects.h>

@interface DASubprocess : NSObject
{
    BOOL	launched;
    BOOL	outAvailable;
    BOOL	errAvailable;
    NSTask	*task;
    NSData	*stdOutData;
    NSData	*stdErrData;
    NSString *exceptionMessage;
}

// launch a series of pipe separated commands
+ (NSDictionary *)launchProcesses:(NSString *)commands;

- (void)launch;
- (void)launchWithInput:(NSString *)input;
- (void)waitUntilExit;

- (NSData *)stdOutData;
- (NSData *)stdErrData;

- (NSDictionary *)resultDictionary;

// covers from NSTask
- (void)setLaunchPath:(NSString *)path;
- (void)setArguments:(NSArray *)arguments;
- (void)setEnvironment:(NSDictionary *)dict;
- (void)setCurrentDirectoryPath:(NSString *)path;

- (BOOL)isRunning;
- (NSString *)launchPath;
- (NSArray *)arguments;
- (NSDictionary *)environment;
- (NSString *)currentDirectoryPath;
- (int)terminationStatus;

- (void)interrupt;
- (void)terminate;

@end
