/* NSTask+Extensions.m created by DAspinall on Fri 21-Aug-1998 */

#import <Foundation/Foundation.h>
#import "NSTask+Extensions.h"

@implementation NSTask (Extensions)

+ (NSDictionary *)performTask:(NSString *)command inDirectory:(NSString *)directory withArgs:(NSArray *)values;
{
    NSTask *task = nil;
    NSMutableDictionary *outputDict = nil;

    NSMutableData	*outData = nil;
    NSPipe			*outPipe = nil;
    NSFileHandle	*outHandle = nil;

    NSMutableData	*errData = nil;
    NSPipe			*errPipe = nil;
    NSFileHandle	*errHandle = nil;
    
    outputDict=[NSMutableDictionary dictionary];
    [outputDict setObject:command forKey:@"LaunchPath"];
    [outputDict setObject:directory forKey:@"CurrentDirectoryPath"];
    [outputDict setObject:values forKey:@"LaunchArguments"];

    outPipe = [NSPipe pipe];
    outHandle = [outPipe fileHandleForReading];
    outData = [NSMutableData data];

    errPipe = [NSPipe pipe];
    errHandle = [errPipe fileHandleForReading];
    errData = [NSMutableData data];

    task = [[NSTask alloc] init];
    [task setCurrentDirectoryPath:directory];
    [task setStandardError:errPipe];
    [task setStandardOutput:outPipe];
    [task setLaunchPath:command];
    [task setArguments:values];

    NS_DURING

    [task launch];

    /* if we are sure that the pipe is running, then we'll cycle while the pipe reads any available data from stderr.  This will only read stuff if the file that we're attempting to decompress is corrupted, or the commandline app that we've launched is horribly verbose (perhaps an argument causes an app to be verbose?)
        */
    if ([task isRunning])
    {
        BOOL	outDone = NO;
        BOOL	errDone = NO;

        int count = 0;
        while ((outDone == NO) || (errDone == NO))
        {
            NSData *inData1 = nil;
            NSData *inData2 = nil;

            NSLog( @"pass %d", count++);
            
            inData1 = [outHandle availableData];
            inData2 = [errHandle availableData];

            if ( inData1 && ([inData1 length] > 0))
                [outData appendData:inData1];
            else
                outDone = YES;

            if ( inData2 && ([inData2 length] > 0))
                [errData appendData:inData2];
            else
                errDone = YES;
        }
    }

    NSLog(@"Out %d Err %d", [outData length], [errData length] );
    [task waitUntilExit];

    [outputDict setObject:[[[NSString alloc] initWithData:errData encoding:NSASCIIStringEncoding] autorelease] forKey:@"StandardError"];
    [outputDict setObject:[[[NSString alloc] initWithData:outData encoding:NSASCIIStringEncoding] autorelease] forKey:@"StandardOutput"];
    [outputDict setObject:[NSNumber numberWithInt:[task terminationStatus]] forKey:@"TerminationStatus"];

    NS_HANDLER
    {
        NSString *msg = [NSString stringWithFormat:@"Caught exception: %@ (%@).", [localException name], [localException reason]];

        NSLog( @"[%s %s] %@",  __FILE__,  sel_getName(_cmd), msg);

        if ( [outputDict objectForKey:@"StandardError"] == nil )
            [outputDict setObject:msg forKey:@"StandardError"];
        else
        {
            [outputDict setObject:[[outputDict objectForKey:@"StandardError"] stringByAppendingFormat:@"\n%@", msg] forKey:@"StandardError"];
        }
        [outputDict setObject:[NSNumber numberWithInt:-1] forKey:@"TerminationStatus"];

    }
    NS_ENDHANDLER

    [task release];

    return outputDict;
}

@end
